@@ -52,13 +52,11 @@ static struct field *parse_field(char *buf, char *given_attr)
 	char *p;
 	char *attr;
 	char *value;
-	int has_attr = 0, len;
+	int has_attr = 0;
 
 	attr = calloc(BUF_SIZE, sizeof(char));
 	value = calloc(BUF_SIZE, sizeof(char));
 	rval = malloc(sizeof(struct field));
-	len = strlen(buf);
-	buf[len - 1] = '\0';
 	if (!rval || !attr || !value) {
 		fprintf(stderr, "Out of memory. Could not allocate memory in parse_field.\n");
 		exit(1);
@@ -82,9 +80,13 @@ static struct field *parse_field(char *buf, char *given_attr)
 			if (*p != ' ') { break; }
 		}
 	} else {
-		strncpy(attr, given_attr, BUF_SIZE);
+		strncpy(attr, given_attr, BUF_SIZE );
 	}
 	strncpy(value, p, BUF_SIZE);
+	if (attr[strlen(attr) - 1] == '\n')
+		attr[strlen(attr) - 1] = '\0';
+	if (value[strlen(value) - 1] == '\n')
+		value[strlen(value) - 1] = '\0';
 	rval->attr = attr;
 	rval->value = value;
 	return rval;
@@ -99,7 +101,7 @@ static struct list *parse_info_file(struct list *l, char *filename, char *given_
 	if (!fd) {
 		return l;
 	}
-	while (fgets(buf, BUF_SIZE, fd)) {
+	while (fgets(buf, BUF_SIZE, fd) != NULL) {
 		struct field *f;
 		f = parse_field(buf, given_attr);
 		if (!f) { continue; }
@@ -148,7 +150,7 @@ static struct list *get_info(char *device_name, int proc_interface)
 	struct list *rval = NULL;
 	struct file_list *list = proc_interface ? proc_list : sys_list;
 	int i, n = (proc_interface ? sizeof(proc_list) : sizeof(sys_list))/sizeof(struct file_list);
-	char *filename = malloc(strlen(device_name) + strlen("/energy_full_design"));
+	char *filename = malloc(strlen(device_name) + strlen("/energy_full_design "));
 
         if (filename == NULL) {
                 fprintf(stderr, "Out of memory. Could not allocate memory in get_info.\n");
@@ -218,14 +220,14 @@ struct list *find_devices(char *acpi_path, int device_nr, int proc_interface)
 	return rval;
 }
 
-static unsigned int get_unit_value(char *value)
+static int get_unit_value(char *value)
 {
 	int n = -1;
 	sscanf(value, "%d", &n);
 	return n;
 }
 
-void print_battery_information(struct list *batteries, int show_empty_slots)
+void print_battery_information(struct list *batteries, int show_empty_slots, int show_capacity)
 {
 	struct list *battery = batteries;
 	struct list *fields;
@@ -236,7 +238,7 @@ void print_battery_information(struct list *batteries, int show_empty_slots)
 		int remaining_capacity = -1;
 		int present_rate = -1;
 		int design_capacity = -1;
-		int real_capacity = -1;
+		int last_capacity = -1;
 		int hours, minutes, seconds;
 		int percentage;
 		char *state = NULL, *poststr;
@@ -247,18 +249,26 @@ void print_battery_information(struct list *batteries, int show_empty_slots)
 			value = fields->data;
 			if (!strcasecmp(value->attr, "remaining capacity")) {
 				remaining_capacity = get_unit_value(value->value);
+				if (!state)
+					state = strdup("available");
 			} else if (!strcmp(value->attr, "charge_now")) {
 				remaining_capacity = get_unit_value(value->value)/1000;
+				if (!state)
+					state = strdup("available");
 			} else if (!strcasecmp(value->attr, "present_rate")) {
 				present_rate = get_unit_value(value->value);
 			} else if (!strcmp(value->attr, "current_now")) {
-				present_rate = abs(get_unit_value(value->value))/1000;
+				present_rate = get_unit_value(value->value)/1000;
 			} else if (!strcasecmp(value->attr, "last full capacity")) {
-				design_capacity = get_unit_value(value->value);
+				last_capacity = get_unit_value(value->value);
+				if (!state)
+					state = strdup("available");
 			} else if (!strcmp(value->attr, "charge_full")) {
-				design_capacity = get_unit_value(value->value)/1000;
+				last_capacity = get_unit_value(value->value)/1000;
+				if (!state)
+					state = strdup("available");
 			} else if (!strcmp(value->attr, "charge_full_design")) {
-				real_capacity = get_unit_value(value->value)/1000;
+				design_capacity = get_unit_value(value->value)/1000;
 			} else if (!strcmp(value->attr, "type")) {
 				type_battery = (strcasecmp(value->value, "battery") == 0);
 			} else if (!strcmp(value->attr, "charging state") ||
@@ -268,25 +278,26 @@ void print_battery_information(struct list *batteries, int show_empty_slots)
 			fields = list_next(fields);
 		}
 		if (type_battery) { /* or else this is the ac_adapter */
-			if (remaining_capacity < 0 || design_capacity < 0 || !state) {
+			if (!state) {
 				if (show_empty_slots) {
 					printf("%12s %d: slot empty\n", BATTERY_DESC, battery_num - 1);
 				}
 			} else {
-				if (design_capacity < MIN_CAPACITY) {
+				if (last_capacity < MIN_CAPACITY) {
 					percentage = 0;
 				} else {
-					percentage = remaining_capacity * 100 / design_capacity;
+					percentage = remaining_capacity * 100 / last_capacity;
 				}
 				if (percentage > 100)
 					percentage = 100;
 				printf("%12s %d: %s, %d%%", BATTERY_DESC, battery_num - 1, state, percentage);
+
 				if (present_rate == -1) {
-					poststr = "rate information unavailable.";
+					poststr = "rate information unavailable";
 					seconds = -1;
 				} else if (!strcasecmp(state, "charging")) {
 					if (present_rate > MIN_PRESENT_RATE) {
-						seconds = 60 * (design_capacity - remaining_capacity) / present_rate;
+						seconds = 3600 * (last_capacity - remaining_capacity) / present_rate;
 						poststr = " until charged";
 					} else {
 						poststr = "charging at zero rate - will never fully charge.";
@@ -294,7 +305,7 @@ void print_battery_information(struct list *batteries, int show_empty_slots)
 					}
 				} else if (!strcasecmp(state, "discharging")) {
 					if (present_rate > MIN_PRESENT_RATE) {
-						seconds = 60 * remaining_capacity / present_rate;
+						seconds = 3600 * remaining_capacity / present_rate;
 						poststr = " remaining";
 					} else {
 						poststr = "discharging at zero rate - will never fully discharge.";
@@ -311,10 +322,25 @@ void print_battery_information(struct list *batteries, int show_empty_slots)
 					minutes = seconds / 60;
 					seconds -= 60 * minutes;
 					printf(", %02d:%02d:%02d%s", hours, minutes, seconds, poststr);
-				} else if ((seconds < 0) && (poststr != NULL)) {
+				} else if (poststr != NULL) {
 					printf(", %s", poststr);
 				}
+
 				printf("\n");
+				
+				if (show_capacity && design_capacity > 0) {
+					if (last_capacity <= 100) {
+						/* some broken systems just give a percentage here */
+						percentage = last_capacity;
+						last_capacity = percentage * design_capacity / 100;
+					} else {
+						percentage = last_capacity * 100 / design_capacity;
+					}
+				if (percentage > 100)
+					percentage = 100;
+					printf("%12s %d: design capacity %d mAh, last full capacity %d mAh = %d%%\n", BATTERY_DESC, battery_num - 1, design_capacity, last_capacity, percentage);
+				}
+
 			}
 			battery_num++;
 		}
@@ -387,17 +413,23 @@ void print_thermal_information(struct list *thermal, int show_empty_slots, int t
 				if (strstr(value->value, "dK")) {
 					temperature = (temperature / 10) - ABSOLUTE_ZERO;
 				}
+				if (!state)
+					state = strdup("available");
 			} else if (!strcmp(value->attr, "sys_temp")) {
 				temperature = get_unit_value(value->value) / 1000.0;
+				if (!state)
+					state = strdup("available");
 			} else if (!strcmp(value->attr, "sys_trip_temp")) {
 				trip_temp = get_unit_value(value->value) / 1000.0;
+				if (!state)
+					state = strdup("available");
 			}
 			fields = list_next(fields);
 		}
 		if (type_zone) { /* or else this is a cooling device */
 			if (temperature < trip_temp) 
 				state = "ok";
-			if (temperature < 0 || !state) {
+			if (!state) {
 				if (show_empty_slots) {
 					printf("%12s %d: slot empty\n", THERMAL_DESC, sensor_num - 1);
 				}
